#include <stdlib.h>
#include "types.h"
#include "animation.h"
#include "tiledata.h"
#include "sound.h"
#include "palettes.h"

/*
	HELPERS
*/

// helper: return 1 if it's time to advance a motion frame, or 0 otherwise
UBYTE advanceMotionFrame(UBYTE time)
{
	//return (time & 7)==0 ? 1 : 0;
	return (time % MOTION_VIDEO_FRAMES_PER_ANIMATION_FRAME)==0 ? 1 : 0;
}

// helper: return 1 if it's time to advance a motion frame, or 0 otherwise
UBYTE advanceMotionFrameCookie(UBYTE time)
{
	//return (time & 7)==0 ? 1 : 0;
	return (time % MOTION_VIDEO_FRAMES_PER_ANIMATION_FRAME_COOKIE)==0 ? 1 : 0;
}

// helper: return 1 if it's time to advance an attacking frame, or 0 otherwise
UBYTE advanceAttackFrame(UBYTE time)
{
	//return (time & 7)==0 ? 1 : 0;
	return (time % ATTACK_VIDEO_FRAMES_PER_ANIMATION_FRAME)==0 ? 1 : 0;
}

/*
	ANIMATE
*/

int burly_animate( burly_t *burly, UBYTE time )
{
	// handle powerup
	if (burly->powerupRemainingFrames > 0)
		burly->powerupRemainingFrames--;
	if (burly->powerupRemainingFrames == 0)
		burly->standardSpeed = BURLY_BASE_SPEED;


	// if we haven't changed state, keep looping through frames
	if (burly->stateMotion == burly->stateMotionOld)
	{
		// if Burly is walking, cycle frame
		if (burly->stateMotion == STATE_BURLY_WALKING )
		{
			burly->totalFrames = ANIMATION_FRAMES_BURLY_WALK_LEFT_OR_RIGHT;
			if(burly->powerupRemainingFrames != 0u)
			{
				burly->currentFrame += advanceMotionFrameCookie(time);
			}
			else
			{
				burly->currentFrame += advanceMotionFrame(time);
			}

			if (burly->totalFrames == burly->currentFrame)
			{
				burly->currentFrame = 0;
				sound_burly_footsteps_ch2();
			}

		}
		// if Burly has been attacking, we need to see if he's done
		if (burly->stateMotion == STATE_BURLY_ATTACKING )
		{
			burly->totalFrames = ANIMATION_FRAMES_BURLY_ATTACK;
			burly->currentFrame += advanceAttackFrame(time);
			if (burly->totalFrames == burly->currentFrame)
			{
				// Burly is done attacking
				burly->stateMotion = STATE_BURLY_STOPPED;
				burly->currentFrame = 0;
			}
		}
	}
	
	// Burly changed state, maybe because of a button press
	if (burly->stateMotion != burly->stateMotionOld) 
	{

		if (burly->stateMotion == STATE_BURLY_ATTACKING )
		{
			burly->speedx.w = 0;
			burly->speedy.w = 0;
			burly->currentFrame = 0;
			burly->totalFrames = ANIMATION_FRAMES_BURLY_ATTACK;
		}
		else if (burly->stateMotion == STATE_BURLY_WALKING )
		{
			burly->currentFrame = 0;
			burly->totalFrames = ANIMATION_FRAMES_BURLY_WALK_LEFT_OR_RIGHT;
		}
	}
	
	return 0;
}

// NOTE: This function will modify the global foxes_remaining value
// by decrementing it ONCE if the fox in question has died and we still have
// to spawn more foxes.
// 
// It has no access to the global, but the value it returns will be stored
// immediately in the global
UBYTE fox_animate( fox_t *fox, UBYTE time, UBYTE remaining )
{
	UBYTE random;

	// if we haven't changed state, keep looping through frames
	if (fox->stateMotion == fox->stateMotionOld)
	{
		//if fox is walking, cycle frame
		if (fox->stateMotion == STATE_FOX_WALKING )
		{
			fox->totalFrames = ANIMATION_FRAMES_FOX_WALK_LEFT_OR_RIGHT;
			fox->currentFrame += advanceMotionFrame(time);
			if (fox->totalFrames == fox->currentFrame)
				fox->currentFrame = 0;
		}
		else if (fox->stateMotion == STATE_FOX_DYING )
		{
			fox->totalFrames = ANIMATION_FRAMES_FOX_DEATH;
			fox->currentFrame += advanceAttackFrame(time);
			if (fox->totalFrames == fox->currentFrame)
			{
				// Fox is dead -- RESPAWN FOX! (if any remain)
				if(remaining > 0)
				{
					fox_initialize(fox, fox->firstSprite, fox->secondSprite);
					remaining--;
				}
				else
				{
					// no more foxes -- set this one to dead
					fox->state = STATE_FOX_DEAD;
				}
			}
			
		}
		else if (fox->stateMotion == STATE_FOX_SITTING )
		{
			fox->totalFrames = ANIMATION_FRAMES_FOX_SITTING;
			fox->currentFrame += advanceMotionFrame(time);
			if (fox->totalFrames == fox->currentFrame)
				fox->currentFrame = 0;
		}
		else if (fox->stateMotion == STATE_FOX_SPAWN )
		{
			fox->totalFrames = ANIMATION_FRAMES_FOX_SPAWNING;
			fox->currentFrame += advanceMotionFrame(time);
			if (fox->totalFrames == fox->currentFrame)
			{
				// fox is done spawning - decide what to do next
				fox->currentFrame = 0;

				// determine a random direction for the fox to start moving
				random = next_random(7u);
				fox->state = get_facing(fox, random);
				fox->speedx.w = get_speedx(fox, random);
				fox->speedy.w = get_speedy(fox, random);

				fox->totalFrames = ANIMATION_FRAMES_FOX_WALK_LEFT_OR_RIGHT;
				fox->stateMotion = STATE_FOX_WALKING;
			}
		}
	}
	
	if (fox->stateMotion != fox->stateMotionOld)
	{
		if (fox->stateMotion == STATE_FOX_DYING)
		{
			fox->speedx.w = 0;
			fox->speedy.w = 0;
			fox->currentFrame = 0;
			fox->totalFrames = ANIMATION_FRAMES_FOX_DEATH;
		}
 		else if (fox->stateMotion == STATE_FOX_WALKING)
		{
			// speed is set before reaching here
			fox->currentFrame = 0;
			fox->totalFrames = ANIMATION_FRAMES_FOX_WALK_LEFT_OR_RIGHT;
		}
		else if (fox->stateMotion == STATE_FOX_SITTING)
		{
			fox->currentFrame = 0;
			fox->speedx.w = 0;
			fox->speedy.w = 0;
			fox->totalFrames = ANIMATION_FRAMES_FOX_SITTING;
		}
		else if (fox->stateMotion == STATE_FOX_SPAWN )
		{
			// since foxes get initialized to STATE_FOX_SPAWN
			// in both stateMotion and stateMotionOld
			// this should never be hit
			fox->currentFrame = 0;
			fox->speedx.w = 0;
			fox->speedy.w = 0;
			fox->totalFrames = ANIMATION_FRAMES_FOX_SPAWNING;
		}
	}
	
	return remaining;
}



/*
	RENDER
*/


int fox_render(fox_t *fox)
{
	UBYTE index = (fox->currentFrame) << 1;

	if (fox->state == STATE_FOX_DEAD)
	{
		set_sprite_tile(fox->firstSprite, tiles_fox_dead);
		set_sprite_tile(fox->secondSprite, tiles_fox_dead);
		set_sprite_prop(fox->firstSprite, 0x00 | fox->palette);
		set_sprite_prop(fox->secondSprite, 0x00 | fox->palette);
		move_sprite(fox->firstSprite, fox->x.b.h, fox->y.b.h);
		move_sprite(fox->secondSprite, fox->x.b.h+8, fox->y.b.h);
		return 0;
	}

	if (fox->stateMotion == STATE_FOX_WALKING)
	{
		// fox sprites are reversed when facing right because they're drawn facing left
		if (fox->state == STATE_FOX_FACING_LEFT)
		{
			set_sprite_tile(fox->firstSprite, fox->walkLeftFrames[index]);
			set_sprite_tile(fox->secondSprite, fox->walkLeftFrames[index+1]);
			set_sprite_prop(fox->firstSprite, 0x00 | fox->palette);
			set_sprite_prop(fox->secondSprite, 0x00 | fox->palette);
		}
		else if (fox->state == STATE_FOX_FACING_RIGHT)
		{
			set_sprite_tile(fox->firstSprite, fox->walkLeftFrames[index+1]);
			set_sprite_tile(fox->secondSprite, fox->walkLeftFrames[index]);
			set_sprite_prop(fox->firstSprite, S_FLIPX | fox->palette);
			set_sprite_prop(fox->secondSprite, S_FLIPX | fox->palette);
		} 
	}
	else if (fox->stateMotion == STATE_FOX_SITTING)
	{
		// fox sprites are reversed when facing right because they're drawn facing left
		if (fox->state == STATE_FOX_FACING_LEFT)
		{
			set_sprite_tile(fox->firstSprite, fox->sittingLeftFrames[index]);
			set_sprite_tile(fox->secondSprite, fox->sittingLeftFrames[index+1]);
			set_sprite_prop(fox->firstSprite, 0x00 | fox->palette);
			set_sprite_prop(fox->secondSprite, 0x00 | fox->palette);
		}
		else if (fox->state == STATE_FOX_FACING_RIGHT)
		{
			set_sprite_tile(fox->firstSprite, fox->sittingLeftFrames[index+1]);
			set_sprite_tile(fox->secondSprite, fox->sittingLeftFrames[index]);
			set_sprite_prop(fox->firstSprite, S_FLIPX | fox->palette);
			set_sprite_prop(fox->secondSprite, S_FLIPX | fox->palette);
		} 
	}
	else if (fox->stateMotion == STATE_FOX_DYING)
	{
		// fox sprites are reversed when facing right because they're drawn facing left
		if (fox->state == STATE_FOX_FACING_LEFT)
		{
			set_sprite_tile(fox->firstSprite, fox->dyingFrames[index]);
			set_sprite_tile(fox->secondSprite, fox->dyingFrames[index+1]);
			set_sprite_prop(fox->firstSprite, 0x00 | fox->palette);
			set_sprite_prop(fox->secondSprite, 0x00 | fox->palette);
		}
		else if (fox->state == STATE_FOX_FACING_RIGHT)
		{
			set_sprite_tile(fox->firstSprite, fox->dyingFrames[index+1]);
			set_sprite_tile(fox->secondSprite, fox->dyingFrames[index]);
			set_sprite_prop(fox->firstSprite, S_FLIPX | fox->palette);
			set_sprite_prop(fox->secondSprite, S_FLIPX | fox->palette);
		} 
	}
	else if (fox->stateMotion == STATE_FOX_SPAWN)
	{
		// fox sprites are reversed when facing right because they're drawn facing left
		if (fox->state == STATE_FOX_FACING_LEFT)
		{
			set_sprite_tile(fox->firstSprite, fox->spawnFrames[index]);
			set_sprite_tile(fox->secondSprite, fox->spawnFrames[index+1]);
			set_sprite_prop(fox->firstSprite, 0x00 | fox->palette);
			set_sprite_prop(fox->secondSprite, 0x00 | fox->palette);
		}
		else if (fox->state == STATE_FOX_FACING_RIGHT)
		{
			set_sprite_tile(fox->firstSprite, fox->spawnFrames[index+1]);
			set_sprite_tile(fox->secondSprite, fox->spawnFrames[index]);
			set_sprite_prop(fox->firstSprite, S_FLIPX | fox->palette);
			set_sprite_prop(fox->secondSprite, S_FLIPX | fox->palette);
		} 
	}

	
	move_sprite(fox->firstSprite, fox->x.b.h, fox->y.b.h);
	move_sprite(fox->secondSprite, fox->x.b.h+8, fox->y.b.h);

	// save state
	fox->stateOld = fox->state;
	fox->stateMotionOld = fox->stateMotion;

	return 0;
}

int burly_render(burly_t *burly)
{
	UBYTE index = (burly->currentFrame) << 1;

	// Burly is attacking
	if (burly->stateMotion == STATE_BURLY_ATTACKING)
	{
		if (burly->state == STATE_BURLY_FACING_RIGHT)
		{
			set_sprite_tile(burly->firstSprite, burly->attackRightFrames[index]);
			set_sprite_tile(burly->secondSprite, burly->attackRightFrames[index+1]);
			set_sprite_prop(burly->firstSprite, 0x00 | PAL_BURLY);
			set_sprite_prop(burly->secondSprite, 0x00 | PAL_BURLY);
		}
		else if (burly->state == STATE_BURLY_FACING_LEFT)
		{
			set_sprite_tile(burly->firstSprite, burly->attackRightFrames[index+1]);
			set_sprite_tile(burly->secondSprite, burly->attackRightFrames[index]);
			set_sprite_prop(burly->firstSprite, S_FLIPX | PAL_BURLY);
			set_sprite_prop(burly->secondSprite, S_FLIPX | PAL_BURLY);
		}
	}
	else
	{
		// Burly is walking or stopped
		if (burly->state == STATE_BURLY_FACING_RIGHT)
		{
			set_sprite_tile(burly->firstSprite, burly->walkRightFrames[index]);
			set_sprite_tile(burly->secondSprite, burly->walkRightFrames[index+1]);
			set_sprite_prop(burly->firstSprite, 0x00 | PAL_BURLY);
			set_sprite_prop(burly->secondSprite, 0x00 | PAL_BURLY);
		}
		else if (burly->state == STATE_BURLY_FACING_LEFT)
		{
			set_sprite_tile(burly->firstSprite, burly->walkRightFrames[index+1]);
			set_sprite_tile(burly->secondSprite, burly->walkRightFrames[index]);
			set_sprite_prop(burly->firstSprite, S_FLIPX | PAL_BURLY);
			set_sprite_prop(burly->secondSprite, S_FLIPX | PAL_BURLY);
		}
	}
	
	move_sprite(burly->firstSprite, burly->x.b.h, burly->y.b.h);
	move_sprite(burly->secondSprite, burly->x.b.h+8, burly->y.b.h);

	// save state
	burly->stateOld = burly->state;
	burly->stateMotionOld = burly->stateMotion;

	return 0;
}

int cookie_render(cookie_t *cookie)
{
	set_sprite_tile(cookie->sprite, cookie->frames);
	set_sprite_prop(cookie->sprite, 0x00 | PAL_COOKIE);
	move_sprite(cookie->sprite, cookie->x.b.h, cookie->y.b.h);

	return 0;
}




//   1  0  7
//   2     6
//   3  4  5
WORD get_speedx(fox_t *fox, UBYTE position)
{
	if(position == 0u)	
		return (WORD)0;
	else if(position == 1u)
		return (WORD)(fox->standardSpeed * (-1));
	else if(position == 2u)
		return (WORD)(fox->standardSpeed * (-1));
	else if(position == 3u)
		return (WORD)(fox->standardSpeed * (-1));
	else if(position == 4u)
		return (WORD)0;
	else if(position == 5u)
		return (WORD)(fox->standardSpeed);
	else if(position == 6u)
		return (WORD)(fox->standardSpeed);
	else if(position == 7u)
		return (WORD)(fox->standardSpeed);
}
WORD get_speedy(fox_t *fox, UBYTE position)
{
	if(position == 0u)	
		return (WORD)(fox->standardSpeed * (-1));
	else if(position == 1u)
		return (WORD)(fox->standardSpeed * (-1));
	else if(position == 2u)
		return (WORD)0;
	else if(position == 3u)
		return (WORD)(fox->standardSpeed);
	else if(position == 4u)
		return (WORD)(fox->standardSpeed);
	else if(position == 5u)
		return (WORD)(fox->standardSpeed);
	else if(position == 6u)
		return (WORD)0;
	else if(position == 7u)
		return (WORD)(fox->standardSpeed * (-1));
}

UBYTE get_facing(fox_t *fox, UBYTE position)
{
	if(position == 0u)
	{
		if(next_random(99u) <= 49u)
			return STATE_FOX_FACING_LEFT;
		else
			return STATE_FOX_FACING_RIGHT;
	}
	else if(position == 1u)
		return STATE_FOX_FACING_LEFT;
	else if(position == 2u)
		return STATE_FOX_FACING_LEFT;
	else if(position == 3u)
		return STATE_FOX_FACING_LEFT;
	else if(position == 4u)
	     {
		if(next_random(99u) <= 49u)
			return STATE_FOX_FACING_LEFT;
		else
			return STATE_FOX_FACING_RIGHT;
	     }
	else if(position == 5u)
		return STATE_FOX_FACING_RIGHT;
	else if(position == 6u)
		return STATE_FOX_FACING_RIGHT;
	else if(position == 7u)
		return STATE_FOX_FACING_RIGHT;
}


