#include <gb/gb.h>

#ifndef SM_TYPES_H
#define SM_TYPES_H

//these are all UBYTEs

#define	ANIMATION_FRAMES_BURLY_WALK_LEFT_OR_RIGHT 4u
#define	ANIMATION_FRAMES_BURLY_ATTACK  4u

// the frame when Burly's swipe is registered
#define	ANIMATION_FRAMES_BURLY_ATTACK_REGISTER_HIT  1u

#define	ANIMATION_FRAMES_FOX_WALK_LEFT_OR_RIGHT 4u
#define	ANIMATION_FRAMES_FOX_DEATH  43u
#define	ANIMATION_FRAMES_FOX_SITTING  1u
#define	ANIMATION_FRAMES_FOX_SPAWNING  9u

#define	STATE_BURLY_WALKING  1u
#define	STATE_BURLY_STOPPED  2u
#define	STATE_BURLY_FACING_LEFT  8u
#define	STATE_BURLY_FACING_RIGHT  16u
#define	STATE_BURLY_ATTACKING  32u

#define	STATE_FOX_WALKING  1u
#define	STATE_FOX_FACING_LEFT  8u
#define	STATE_FOX_FACING_RIGHT  16u
#define STATE_FOX_SITTING 32u
// once fox reaches the last frame of the dying animation,
// its coordinates are reset to random values, respawning the fox at a random location
#define	STATE_FOX_DYING  64u
#define STATE_FOX_SPAWN  50u
#define	STATE_FOX_DEAD  100u

// how frequently we update the animations of characters
#define MOTION_VIDEO_FRAMES_PER_ANIMATION_FRAME 3u
#define MOTION_VIDEO_FRAMES_PER_ANIMATION_FRAME_COOKIE 2u
#define ATTACK_VIDEO_FRAMES_PER_ANIMATION_FRAME 2u

// Burly's speed when no powerups are in effect
#define BURLY_BASE_SPEED 0x0150u;
// by how much we increase Burly's speed when a cookie is picked up
#define BURLY_SPEED_POWERUP_INCREMENT 0x0150u;
// how many frames does a powerup last?
#define BURLY_POWERUP_DURATION 65u;

// total number of foxes
#define MAX_FOXES 8u

// burly's struct
typedef struct
{
	// position
	fixed x, y;

	// speed (can be negative)
	fixed speedx, speedy;	

	// these pertain only to the current animation
        UBYTE totalFrames;
	UBYTE currentFrame;

	// tells which way Burly is facing
	UBYTE state;
	UBYTE stateOld;
	// tells whether Burly is stopped, walking, or attacking
	UBYTE stateMotion;
	UBYTE stateMotionOld;

	// global number of burly's first and second sprites
	UBYTE firstSprite;
	UBYTE secondSprite;

	// this can increase when a cookie is eaten
	WORD standardSpeed;

	// when burly picks up a honey jar or a cookie, this receives a 
	// positive value; it is decremented every frame, until it reaches 0 again
	// when it reaches 0, burly returns to normal
	UBYTE powerupRemainingFrames;

	unsigned char *walkRightFrames;
	unsigned char *walkUpFrames;
	unsigned char *walkDownFrames;

	unsigned char *attackRightFrames;
	unsigned char *attackUpFrames;
	unsigned char *attackDownFrames;

} burly_t;
	

// fox's struct
typedef struct
{
	// position
	fixed x, y;

	// speed (can be negative)
	fixed speedx, speedy;	

	// these pertain only to the current animation
	UBYTE currentFrame;
        UBYTE totalFrames;
	
	// tells which way the fox is facing
	UBYTE state;
	UBYTE stateOld;
	// tells if the fox is stopped, walking, or dying
	UBYTE stateMotion;
	UBYTE stateMotionOld;

	// Unless this useless variable is defined, the variable UBYTE stateMotionOld above will 
	// cause the program to crash or act strangely merely if it is defined
	// I have no idea what's going on -- perhaps some sort of compiler bug dealing with
	// memory alignment since the symptom of the crash or strange behaviour changes with the amount
	// of foxes defined (which in turn causes different amounts of memory to be allocated for the 
	// array of foxes .. sometimes it crashes with "invalid opcode", sometimes the screen corrupts.
	//
	// Try commenting out the variable below and see what happens! :D
	//
	UBYTE useless_not_used_anywhere1;

	// global number of this fox's first and second sprites
	UBYTE firstSprite;
	UBYTE secondSprite;

	// palette (foxes can have different colours)
	UBYTE palette;
	
	WORD standardSpeed;

	unsigned char *walkLeftFrames;
	unsigned char *sittingLeftFrames;
	unsigned char *dyingFrames;
	unsigned char *spawnFrames;

	unsigned char *walkDownFrames;
	unsigned char *attackUpFrames;
	unsigned char *attackDownFrames;

} fox_t;

// cookie
typedef struct
{
	// position
	fixed x, y;

	// global number of burly's first and second sprites
	UBYTE sprite;

	unsigned char *frames;

} cookie_t;


// initialize burly
int burly_initialize( burly_t *burly );

// initialize fox
int fox_initialize( fox_t *fox, UBYTE first, UBYTE second );

// initialize cookie
int cookie_initialize( cookie_t *cookie, UBYTE first );

// initialize random number generator
int seed_random(UBYTE seed);

// get a random number between 0 and n inclusive
UBYTE next_random(UBYTE n);

#endif
