/*
 *
 *  -------------------------------
 * | Burly Bear vs. The Mean Foxes |
 *  -------------------------------
 *
 *         based loosely on the Romanian folk tale of how 
 *         the bear lost his tail because of the fox's trickery
 *
 * written by Sebastian Mihai
 * http://www.sebastianmihai.com
 * 2011
 *
 *
 * This short little game is a self-teaching tool and is not meant to be production-quality.
 * It's only slightly optimized, and is, instead, written in such a way that is quite readable
 * (try going through some other lower-level C programs, especially games written for older 
 * video game systems and notice how cryptic they can be!)
 * 
 * I started from the GBDK example called galaxy.c, which demonstrates basic concepts of
 * sprite, windows, and background scrolling and animation on the Gameboy. The first step was to 
 * weed out everything I wouldn't need, including the window. I then removed most of the animations 
 * and sprite data, in order to add my own. 
 *
 * As far as the graphics go, I wanted to see if I could make a good enough sprite/tile maker
 * in Excel. You can find it in the package you downloaded; I used it exclusively for this game.
 *
 * Feel free to take this program and do whatever you want with it. If you use it as a starting
 * point and make a nice game, or an animation, or whatever, I would love to see it too! See my
 * website above for details on how to send it to me. The compiled .gb file is enough! :)
 *
 * Enjoy!
 * 		Sebastian
 *
 *
 */

const char manyblanks[] = "                                                                                                                                                                                                                                                  ";
const char msg[] = "Burly Bear vs. The Mean Foxes";
const char manyblanks2[] = "                                                                                                                                                                                                                                                  ";
const char msg2[] = "Written by Sebastian Mihai, 2011";
const char msg3[] = "           http://www.sebastianmihai.com      (this website was valid in 2011...not sure about now)         ";
const char msg4[] = " This short little game is a self-teaching tool and is not meant to be production-quality. It's only slightly optimized, and is, instead, written in such a way that is quite readable (try going through some other lower-level C programs, especially games written for older video game systems and notice how cryptic they can be!)        I started from the GBDK example called galaxy.c, which demonstrates basic concepts of sprite, windows, and background scrolling and animation on the Gameboy. The first step was to weed out everything I wouldn't need, including the window. I then removed most of the animations and sprite data, in order to add my own.        As far as the graphics go, I wanted to see if I could make a good enough sprite/tile maker in Excel. You can find it in the package you downloaded; I used it exclusively for this game.       Feel free to take this program and do whatever you want with it. If you use it as a starting point and make a nice game, or an animation, or whatever, I would love to see it too! See my website above for details on how to send it to me. The compiled .gb file is enough! :)";

const char manyblanks3[] = "                                                                                                                                                                                                                                                  ";



#include <gb/gb.h>
#include "tiledata.h"
#include "types.h"
#include "animation.h"
#include "gamelogic.h"
#include "sound.h"


// background objects
#define BG_ITEM_NOTHING 0u
#define	BG_ITEM_GRASS1 1u
#define BG_ITEM_GRASS2 2u
#define BG_ITEM_GRASS3 3u
#define BG_ITEM_ROCK 4u
#define BG_ITEM_TREE1_BOTTOMRIGHT 5u
#define BG_ITEM_TREE2_BOTTOMRIGHT 6u
#define BG_ITEM_STUMP 7u
#define BG_ITEM_LOGS 8u
#define BG_ITEM_MUSHROOM 9u

// level creation selectors
#define LEVEL_GRASSY_ROCKY_HILL 1u
#define LEVEL_MOUNTAIN_FOREST 2u
#define LEVEL_HAUNTED_GLADE 3u
#define LEVEL_FERTILE_FIELDS 4u
#define LEVEL_LOWLAND_FOREST 5u


// global frame counter
static UBYTE time;

// background position and speed
fixed bposx, bposy;

static UBYTE palletteSelector;

//
// Simulate movement by scrolling the background and everything else around Burly
// who is stationary at all times
//
void scroll()
{
	UBYTE i;
	WORD minusburlyspeedx = (-1)*burly.speedx.w;
	WORD minusburlyspeedy = (-1)*burly.speedy.w;

	// scroll background
	bposx.w += burly.speedx.w;
	bposy.w += burly.speedy.w;
	SCX_REG = bposx.b.h;
	SCY_REG = bposy.b.h;

	// shift all foxes
	// (even the dying ones)
	for(i=0;i<MAX_FOXES;i++)
	{
		// move foxes according to burly's speed (background scrolling speed)
		fox[i].x.w += minusburlyspeedx;
		fox[i].y.w += minusburlyspeedy;
	}

	// shift cookie too
	cookie.x.w += minusburlyspeedx;
	cookie.y.w += minusburlyspeedy;
}

unsigned char* get_tile_for_background (UBYTE tile)
{	
	unsigned char* ret;
	if(tile == BG_ITEM_NOTHING)
	{
		ret =  tiles_nothing;
	}
	else if(tile == BG_ITEM_GRASS1)
	{
		ret = tiles_grass;
	}
	else if(tile == BG_ITEM_GRASS2)
	{
		ret = tiles_grass2;
	}
	else if(tile == BG_ITEM_GRASS3)
	{
		ret = tiles_grass3;
	}
	else if(tile == BG_ITEM_ROCK)
	{
		ret = tiles_rock;
	}
	else if(tile == BG_ITEM_TREE1_BOTTOMRIGHT)
	{
		ret = tiles_tree;
	}
	else if(tile == BG_ITEM_TREE2_BOTTOMRIGHT)
	{
		ret = tiles_tree2;
	}
	else if(tile == BG_ITEM_MUSHROOM)
	{
		ret = tiles_mushroom;
	}

	return ret;
}

UBYTE random_tile(UBYTE levelSelector)
{
	UBYTE r = next_random(200);

	UBYTE tier1, tier2, tier3, tier4, tier5;

	// probability of occurrence, in 1/200 units
	UBYTE rocks;
	UBYTE grass1;
	UBYTE grass2;
	UBYTE grass3;
	UBYTE mushrooms;

	if(levelSelector == LEVEL_GRASSY_ROCKY_HILL)
	{
		rocks = 2u;
		grass1 = 2u;
		grass2 = 2u;
		grass3 = 2u;
		mushrooms = 3u;
	}
	else if(levelSelector == LEVEL_MOUNTAIN_FOREST)
	{
		rocks = 5u;
		grass1 = 5u;
		grass2 = 0u;
		grass3 = 10u;
		mushrooms = 0u;
	}
	else if(levelSelector == LEVEL_HAUNTED_GLADE)
	{
		rocks = 5u;
		grass1 = 0u;
		grass2 = 0u;
		grass3 = 0u;
		mushrooms = 1u;
	}
	else if(levelSelector == LEVEL_FERTILE_FIELDS)
	{
		rocks = 0u;
		grass1 = 7u;
		grass2 = 7u;
		grass3 = 7u;
		mushrooms = 7u;
	}
	else if(levelSelector == LEVEL_LOWLAND_FOREST)
	{
		rocks = 2u;
		grass1 = 2u;
		grass2 = 2u;
		grass3 = 2u;
		mushrooms = 2u;
	}

	tier1 = rocks;
	tier2 = tier1 + grass1;
	tier3 = tier2 + grass2;
	tier4 = tier3 + grass3;
	tier5 = tier4 + mushrooms;



	if( r >= 0u && r < tier1 )
		return BG_ITEM_ROCK;
	else if( r >= tier1 && r < tier2 )
		return BG_ITEM_GRASS1;
	else if( r >= tier2 && r < tier3 )
		return BG_ITEM_GRASS2;
	else if( r >= tier3 && r < tier4 )
		return BG_ITEM_GRASS3;
	else if( r >= tier4 && r < tier5 )
		return BG_ITEM_MUSHROOM;

	
	return BG_ITEM_NOTHING;
}

UBYTE random_treetile(UBYTE levelSelector)
{
	UBYTE r = next_random(200);

	UBYTE tier1, tier2, tier3, tier4;

	UBYTE tree1;
	UBYTE tree2;
	UBYTE stump;
	UBYTE logs;

	if(levelSelector == LEVEL_GRASSY_ROCKY_HILL)
	{
		tree1 = 0u;
		tree2 = 0u;
		stump = 0u;
		logs = 0u;
	}
	else if(levelSelector == LEVEL_MOUNTAIN_FOREST)
	{
		tree1 = 30u;
		tree2 = 0u;
		stump = 3u;
		logs = 5u;
	}
	else if(levelSelector == LEVEL_HAUNTED_GLADE)
	{
		tree1 = 0u;
		tree2 = 0u;
		stump = 40u;
		logs = 0u;
	}
	else if(levelSelector == LEVEL_FERTILE_FIELDS)
	{
		tree1 = 0u;
		tree2 = 0u;
		stump = 0u;
		logs = 0u;
	}
	else if(levelSelector == LEVEL_LOWLAND_FOREST)
	{
		tree1 = 0u;
		tree2 = 30u;
		stump = 20u;
		logs = 20u;
	}

	tier1 = tree1;
	tier2 = tier1 + tree2;
	tier3 = tier2 + stump;
	tier4 = tier3 + logs;


	if( r >= 0u && r < tier1 )
		return BG_ITEM_TREE1_BOTTOMRIGHT;
	else if( r >= tier1 && r < tier2 )
		return BG_ITEM_TREE2_BOTTOMRIGHT;
	else if( r >= tier2 && r < tier3 )
		return BG_ITEM_STUMP;
	else if( r >= tier3 && r < tier4 )
		return BG_ITEM_LOGS;
	
	return BG_ITEM_NOTHING;
}


// randomly place objects on the background
void generate_background(UBYTE levelSelector)
{
	UBYTE i, j;
	UBYTE tile;

	// tile pass
	for(i = 0; i < 32; i+=1)
	{
		for(j = 0; j < 32; j+=1)
		{
			tile = random_tile(levelSelector);
			set_bkg_tiles(i, j, 1, 1, get_tile_for_background(tile));
		}
	}
	
	// tree pass
	for(i = 1; i < 32; i+=2)
	{
		for(j = 1; j < 32; j+=2)
		{
			// sometimes shift horizontally so trees are not perfectly aligned
			// do this every four rows only, so stuff doesn't get shifted back to
			// the original column, aligning trees too "artificially"
			if(i<31 && next_random(4) == 0)
				i++;

			tile = random_treetile(levelSelector);
			if(tile == BG_ITEM_TREE1_BOTTOMRIGHT)
			{
				set_bkg_tiles(i-1, j-1, 2, 2, tiles_tree );
			}
			else if(tile == BG_ITEM_TREE2_BOTTOMRIGHT)
			{
				set_bkg_tiles(i-1, j-1, 2, 2, tiles_tree2 );
			}
			else if(tile == BG_ITEM_STUMP)
			{
				set_bkg_tiles(i-1, j, 2, 1, tiles_stump );
			}
			else if(tile == BG_ITEM_LOGS)
			{
				set_bkg_tiles(i-1, j, 2, 1, tiles_logs );
			}
		}
	}

}



//
// Called before anything meaningful is done with the screen
//
//
void load_registers_and_graphics()
{
	UBYTE i, j;

	disable_interrupts();
	DISPLAY_OFF;
	LCDC_REG = 0x67; //sprites are 8x16 -- use 0x63 for 8x8 sprites

	// palettes
	if(palletteSelector == 0u)
	{
		BGP_REG = OBP0_REG = OBP1_REG = 0x27U;
	}
	else
	{
		OBP0_REG = OBP1_REG = 0xE4U;
		BGP_REG = 0xE4U;
	}


	// load background data
	set_bkg_data(0xFC, 0x04, std_data);
	set_bkg_data(0x00, 0x15, data_main_menu);
	set_bkg_data(0x15, 0x11, data_background);
	set_bkg_data(0x26, 0x03, data_sm_2011);

	// setup background
	bposx.w = 0;
	SCX_REG = 0;
	bposy.w = 0;
	SCY_REG = 0;
	WX_REG = MAXWNDPOSX;
	WY_REG = MAXWNDPOSY;

	// load sprite data
	// first param = starting location
	// second param = count
	set_sprite_data(0x00, 0x0C, data_burly_walk_right);
	set_sprite_data(0x0C, 0x10, data_fox_walk_left);
	set_sprite_data(0x1C, 0x04, data_fox_sitting);
	set_sprite_data(0x20, 0x10, data_burly_attack_right);
	set_sprite_data(0x30, 0x14, data_fox_dying_left);
	set_sprite_data(0x44, 0x08, data_fox_spawn);
	set_sprite_data(0x4C, 0x02, data_cookie);
	set_sprite_data(0x4E, 0x02, data_fox_dead);

	for(i = 0; i < 32; i+=1)
	{
		for(j = 0; j < 32; j+=1)
		{
			set_bkg_tiles(i, j, 1, 1, tiles_flat_white);
		}
	}

	DISPLAY_ON;
	enable_interrupts();

	// begin with a black background
	for(i = 0; i < 32; i+=1)
	{
		for(j = 0; j < 32; j+=1)
		{
			set_bkg_tiles(i, j, 1, 1, tiles_nothing);
		}
		wait_vbl_done();
	}
}

//
//
// Show a menu and wait for START to be pressed
//
//
UBYTE menu()
{
	UBYTE i, j;

	load_registers_and_graphics();


	for(i = 0; i < 32; i+=2)
	{
		set_bkg_tiles(i, 1, 1, 1, tiles_grass2);
		set_bkg_tiles(i+1, 1, 1, 1, tiles_grass);
		
		set_bkg_tiles(i, 17, 1, 1, tiles_grass2);
		set_bkg_tiles(i+1, 17, 1, 1, tiles_grass);
	}
	set_bkg_tiles(0, 0, 2, 2, tiles_tree2);
	set_bkg_tiles(9, 0, 2, 2, tiles_tree);
	set_bkg_tiles(18, 0, 2, 2, tiles_tree2);
	
	set_bkg_tiles(0, 16, 2, 2, tiles_tree);
	set_bkg_tiles(9, 16, 2, 2, tiles_tree2);
	set_bkg_tiles(18, 16, 2, 2, tiles_tree);
	
	set_bkg_tiles(2, 5, 0x0F, 1, tiles_main_menu);
	set_bkg_tiles(7, 9, 0x06, 1, tiles_main_menu_line2);

	set_bkg_tiles(16, 15, 3, 1, tiles_sm_2011);

	// set-up characters on the menu screen
	burly_initialize(&burly);
	for(i=0;i<MAX_FOXES;i++)
	{
		fox_initialize(&fox[i], 2 + (i * 2), 2 + (i * 2 + 1) );
	}
	burly.x.w = 0x2000;
	burly.y.w = 0x5500;
	burly_render(&burly);
	i=0;j=0;
	for(i = 0; i < 2; i+=1)
	{	
		if(i*2+j == MAX_FOXES) break;
		for(j = 0; j < 2; j+=1)
		{
			if(i*2+j == MAX_FOXES) break;
			fox[i*2+j].x.w = 0x7800 + (j*0x1001);
			fox[i*2+j].y.w = 0x4F00 + (i*0x1001);
			fox[i*2+j].stateMotion = STATE_FOX_SITTING;
			fox[i*2+j].state = STATE_FOX_FACING_LEFT;
			fox_render(&fox[i*2+j]);
		}
	}

	// wait for user to release START and SELECT buttons
	while((joypad() & J_START) || (joypad() & J_SELECT))
	{
	}

	// WAIT FOR START TO BE PRESSED
	// If SELECT is pressed, give new value to palletteSelector and exit
	while(!(joypad() & J_START))
	{
		if(joypad() & J_SELECT)
		{
			palletteSelector++;
			if(palletteSelector>1u)
				palletteSelector=0u;

			return 1u; // just changed pallette - reload menu!
		}
		time++;
		if(time%255==0)
			sound_waiting_for_enter_ch1(time);
	}

	sound_enter_pressed_ch1();

	// move sprites offscreen
	burly.x.w = 0xD000;
	burly.y.w = 0xD000;
	burly_render(&burly);
	i=0;j=0;
	for(i = 0; i < 2; i+=1)
	{	
		if(i*2+j == MAX_FOXES) break;
		for(j = 0; j < 2; j+=1)
		{
			if(i*2+j == MAX_FOXES) break;
			fox[i*2+j].x.w = 0xD000;
			fox[i*2+j].y.w = 0xD000;
			fox_render(&fox[i*2+j]);
		}
	}

	// seed at end of menu
	seed_random(time);
	return 0u;
}

void write_nice_on_screen(unsigned char *tile)
{
	UBYTE i = 1;

	set_bkg_tiles(i, 4, 0x01, 1, tile);
	set_bkg_tiles(i, 5, 0x01, 1, tile);
	set_bkg_tiles(i, 6, 0x01, 1, tile);
	set_bkg_tiles(i, 7, 0x01, 1, tile);
	set_bkg_tiles(i, 8, 0x01, 1, tile);
	set_bkg_tiles(i+1, 5, 0x01, 1, tile);
	set_bkg_tiles(i+2, 6, 0x01, 1, tile);
	set_bkg_tiles(i+3, 7, 0x01, 1, tile);

	set_bkg_tiles(i+4, 4, 0x01, 1, tile);
	set_bkg_tiles(i+4, 5, 0x01, 1, tile);
	set_bkg_tiles(i+4, 6, 0x01, 1, tile);
	set_bkg_tiles(i+4, 7, 0x01, 1, tile);
	set_bkg_tiles(i+4, 8, 0x01, 1, tile);
	
	set_bkg_tiles(i+6, 4, 0x01, 1, tile);
	set_bkg_tiles(i+6, 5, 0x01, 1, tile);
	set_bkg_tiles(i+6, 6, 0x01, 1, tile);
	set_bkg_tiles(i+6, 7, 0x01, 1, tile);
	set_bkg_tiles(i+6, 8, 0x01, 1, tile);
	
	set_bkg_tiles(i+9, 4, 0x01, 1, tile);
	set_bkg_tiles(i+8, 5, 0x01, 1, tile);
	set_bkg_tiles(i+8, 6, 0x01, 1, tile);
	set_bkg_tiles(i+8, 7, 0x01, 1, tile);
	set_bkg_tiles(i+9, 8, 0x01, 1, tile);

	set_bkg_tiles(i+10, 4, 0x01, 1, tile);
	set_bkg_tiles(i+10, 8, 0x01, 1, tile);
	set_bkg_tiles(i+11, 5, 0x01, 1, tile);
	set_bkg_tiles(i+11, 7, 0x01, 1, tile);

	set_bkg_tiles(i+13, 4, 0x01, 1, tile);
	set_bkg_tiles(i+13, 5, 0x01, 1, tile);
	set_bkg_tiles(i+13, 6, 0x01, 1, tile);
	set_bkg_tiles(i+13, 7, 0x01, 1, tile);
	set_bkg_tiles(i+13, 8, 0x01, 1, tile);
	
	set_bkg_tiles(i+14, 4, 0x01, 1, tile);
	set_bkg_tiles(i+14, 6, 0x01, 1, tile);
	set_bkg_tiles(i+14, 8, 0x01, 1, tile);
	set_bkg_tiles(i+15, 4, 0x01, 1, tile);
	set_bkg_tiles(i+15, 6, 0x01, 1, tile);
	set_bkg_tiles(i+15, 8, 0x01, 1, tile);

	set_bkg_tiles(i+17, 4, 0x01, 1, tile);
	set_bkg_tiles(i+17, 5, 0x01, 1, tile);
	set_bkg_tiles(i+17, 6, 0x01, 1, tile);
	set_bkg_tiles(i+17, 8, 0x01, 1, tile);
}

//
//
// This is called in-between levels
//
//
void next_level()
{
	UBYTE i = 0;

	load_registers_and_graphics();
	
	// wait for user to release START button
	while((joypad() & J_START))
	{
		time++;
	}

	// These are "do nothing" statements -- they are here
	// as a hack to counter various memory alignment
	// problems I've run into
	// Removing any of them may cause undesired behaviour
	set_bkg_tiles(0, 0, 0x01, 1, tiles_nothing);
	set_bkg_tiles(1, 0, 0x01, 1, tiles_nothing);
	set_bkg_tiles(1, 0, 0x01, 1, tiles_nothing);
	//set_bkg_tiles(1, 0, 0x01, 1, tiles_nothing);
	//set_bkg_tiles(1, 0, 0x01, 1, tiles_nothing);
	//set_bkg_tiles(1, 0, 0x01, 1, tiles_nothing);


	set_bkg_tiles(7, 11, 0x06, 1, tiles_main_menu_line2);

	// WAIT FOR START TO BE PRESSED
	while(!(joypad() & J_START))
	{
		time++;

		if(time % 32 < 16)
		{
			write_nice_on_screen(tiles_grass);
		}
		else
		{
			write_nice_on_screen(tiles_grass2);
		}

	}
	sound_enter_pressed_ch1();
	// seed at end of menu
	seed_random(time);
}

// Create foxes based on foxes_remaining and MAX_FOXES
// if less foxes than MAX_FOXES are needed, the array from
// [foxes_remaining] to [MAX_FOXES-] will contain dead foxes
// upon return, foxes_remaining is set to how many more will need to spawn
void initialize_all_foxes()
{
	UBYTE i;

	for(i=0;i<MAX_FOXES;i++)
	{
		if(i<foxes_remaining)
			fox_initialize(&fox[i], 2 + (i * 2), 2 + (i * 2 + 1) );
	}
	for(i=foxes_remaining;i<MAX_FOXES;i++)
	{
		fox_initialize(&fox[i], 2 + (i * 2), 2 + (i * 2 + 1) );
		fox[i].state = STATE_FOX_DEAD;
	}
	
	if(foxes_remaining <= MAX_FOXES)
		foxes_remaining = 0;
	else
		foxes_remaining -= MAX_FOXES;
}

// return 1 if Burly has defeated all foxes on this level
// return 0 otherwise
UBYTE victory_accomplished()
{
	UBYTE i;

	if (foxes_remaining > 0)
		return 0;

	for(i=0;i<MAX_FOXES;i++)
	{
		if(fox[i].state != STATE_FOX_DEAD)
			return 0;
	}

	return 1;
}

UBYTE get_fox_count_for_level(UBYTE levelSelector)
{
	if(levelSelector == LEVEL_GRASSY_ROCKY_HILL)
		return 15u;
	else if(levelSelector == LEVEL_MOUNTAIN_FOREST)
		return 20u;
	else if(levelSelector == LEVEL_HAUNTED_GLADE)
		return 8u;
	else if(levelSelector == LEVEL_FERTILE_FIELDS)
		return 15u;
	else if(levelSelector == LEVEL_LOWLAND_FOREST)
		return 20u;

	return 0u;
}

//
//
// Run a level based on a selector
//
//
void game(UBYTE levelSelector)
{
	UBYTE i;
	WORD a, b;

	load_registers_and_graphics();
	DISPLAY_OFF;
	generate_background(levelSelector);
	DISPLAY_ON;

	foxes_remaining = get_fox_count_for_level(levelSelector);

	// initialize Burly, foxes, and cookie
	burly_initialize(&burly);
	initialize_all_foxes();
	cookie_initialize(&cookie, 2+MAX_FOXES*2);

	if (levelSelector == LEVEL_HAUNTED_GLADE)
	{
		for(i=0;i<MAX_FOXES;i++)
		{
			if(fox[i].state != STATE_FOX_DEAD)
			{
				a = 0x00A0u;
				b = fox[i].standardSpeed;
				fox[i].standardSpeed = a + b;
			}
		}
	}

	// MAIN GAME LOOP!
	while(!victory_accomplished())
	{
		// skip a few VBLs to slow down animation if needed
		for(i = 0; i < 0; i++)  wait_vbl_done();

		// increment global frame counter
		time++;

		animate(time);
		scroll();
		render();
		handle_input(&burly, time);
		interactions();
		ai();
	}
	
	// remove Burly and cookie from screen before the closing of the level
	set_sprite_tile(0, tiles_fox_dead);
	set_sprite_tile(1, tiles_fox_dead);
	set_sprite_tile(cookie.sprite, tiles_fox_dead);
}

// program entry point
void main()
{
	// Global time counter
	time = 0u;
	
	// 0u == dark, 1u == white
	palletteSelector = 0u;

	// beeps and blips
	sound_init();

	while(1)
	{
		// if the call to menu() returns something else than 0, 
		// we're merely changing the pallette registers, and then
		// displaying the menu again
		while(menu()!=0u)
		{
			time = 0u;
		}

		game(LEVEL_GRASSY_ROCKY_HILL);
		next_level();
		game(LEVEL_MOUNTAIN_FOREST);
		next_level();
		game(LEVEL_FERTILE_FIELDS);
		next_level();
		game(LEVEL_HAUNTED_GLADE);
		next_level();
		game(LEVEL_LOWLAND_FOREST);
	}	
}
