#include <gb/gb.h>
#include <stdlib.h>
#include "gamelogic.h"
#include "types.h"
#include "sound.h"

// protagonist bear Burly
burly_t burly;

// mean foxes
fox_t fox[MAX_FOXES];

// cookie power-up
cookie_t cookie;

// how many foxes left until level is completed
UBYTE foxes_remaining;

/*
	HITTING
*/
// return 1 if Burly is in a position to swipe successfully at this fox
// return 0 otherwise
UBYTE in_swipe_range(burly_t *burly, fox_t *fox)
{
	if(abs(burly->y.w - fox->y.w) <= 0x0A00)
	{
		if( burly->x.w >= fox->x.w && burly->state == STATE_BURLY_FACING_LEFT )
		{
			if( (burly->x.w-fox->x.w<=0x1200 && fox->state == STATE_FOX_FACING_LEFT) ||
			    (burly->x.w-fox->x.w<=0x1300 && fox->state == STATE_FOX_FACING_RIGHT) )
			{
				return 1u;
			}
		}
		else if(burly->x.w < fox->x.w && burly->state == STATE_BURLY_FACING_RIGHT)
		{
			if( (fox->x.w-burly->x.w<=0x1300 && fox->state == STATE_FOX_FACING_LEFT) ||
			    (fox->x.w-burly->x.w<=0x1200 && fox->state == STATE_FOX_FACING_RIGHT) )
			{
				return 1u;
			}
		}
	}

	return 0u;
}

//
// return 1 if Burly is near the cookie
// return 0 otherwise
//
UBYTE in_cookie_range(burly_t *burly, cookie_t *cookie)
{
	// Y hitbox is different for when Burly is above or below
	if((abs(burly->y.w - cookie->y.w) <= 0x0D00 && burly->y.w < cookie->y.w) ||
           (abs(burly->y.w - cookie->y.w) <= 0x0300 && burly->y.w >= cookie->y.w))
	{
		if((abs(burly->x.w - cookie->x.w) <= 0x0900 && burly->x.w < cookie->x.w) ||
           	   (abs(burly->x.w - cookie->x.w) <= 0x0300 && burly->x.w >= cookie->x.w))
		{
			return 1u;
		}
	}

	return 0u;
}

//
// Make fox think a bit
// 
//
int fox_ai(fox_t *fox)
{
	UBYTE random;
	// initial AI short-circuit (state independent)

	// if this fox is dying or spawning, it can't really change what it is doing
	if(fox->stateMotion == STATE_FOX_DYING || fox->stateMotion == STATE_FOX_SPAWN)
	{
		return 0;
	}

	// a very high chance to not change what the fox is currently doing
	if(next_random(99u) <= 95u)
	{
		return 0;
	}

	// secondary AI short-circuit (state dependent)

	if(fox->stateMotion == STATE_FOX_SITTING)
	{
		// if fox is sitting, a good chance to continue sitting	
		if(next_random(99u) <= 49u)
		{
			return 0;
		}
		else
		{
			// and 50% chance to start walking in a random direction
			
			// determine direction
			//   1  0  7
			//   2     6
			//   3  4  5
			random = next_random(7u);
			fox->stateMotion = STATE_FOX_WALKING;
			fox->state = get_facing(fox, random);
			fox->speedx.w = get_speedx(fox, random);
			fox->speedy.w = get_speedy(fox, random);
			return 0;
		}
	}
	else if(fox->stateMotion == STATE_FOX_WALKING)
	{
		// if fox is walking, 80% chance to continue walking	
		if(next_random(99u) <= 79u)
		{
			return 0;
		}
		else
		{
			// else to sit down
			fox->stateMotion = STATE_FOX_SITTING;
			return 0;
		}
	}
	
}

//
// Interactions between characters, item pick-up, etc.
//
UBYTE interactions()
{
	UBYTE i;
	WORD a;
	UBYTE anyHit = 0u;
	// register hit on foxes
	if(burly.stateMotion == STATE_BURLY_ATTACKING)
	//&& burly.currentFrame == ANIMATION_FRAMES_BURLY_ATTACK_REGISTER_HIT)
	{
		// loop through foxes and see which ones were hit
		for(i=0;i<MAX_FOXES;i++)
		{
			if(fox[i].state != STATE_FOX_DEAD)
			{
				if(in_swipe_range(&burly, &fox[i]) && (fox[i].stateMotion!=STATE_FOX_DYING))
				{
					fox[i].stateMotion = STATE_FOX_DYING;
					anyHit = 1u;
				}
			}
		}
		if (anyHit)
		{
			sound_fox_hit_ch1();
		}
	}

	// pick up cookie if near it
	if(in_cookie_range(&burly, &cookie))
	{
		// reset cookie timer to maximum 
		burly.powerupRemainingFrames = BURLY_POWERUP_DURATION;

		// increase Burly's speed
		// 
		// This is a hack - lcc would just die if this intermediary
		// variable isn't used.
		// Fun!
		a = burly.standardSpeed;
		a = a + BURLY_SPEED_POWERUP_INCREMENT;
		burly.standardSpeed = a;

		cookie_initialize(&cookie, cookie.sprite);
		sound_cookie_pickup_ch1();
	}

	return 0u;
}

//
// Fox AI
//
UBYTE ai()
{
	UBYTE i;

	// fox ai
	for(i=0;i<MAX_FOXES;i++)
	{
		if(fox[i].state != STATE_FOX_DEAD)
			fox_ai(&fox[i]);
	}

	return 0u;
}


//
// Calculate current animation frames, and state changes
//
UBYTE animate(const UBYTE time)
{
	UBYTE i;

	burly_animate( &burly, time);
	for(i=0;i<MAX_FOXES;i++)
	{
		if(fox[i].state != STATE_FOX_DEAD)
			foxes_remaining = fox_animate(&fox[i], time, foxes_remaining);
	}
	return 0u;
}

//
// Calculate what sprites are shown for each character, and move them in the video memory
// according to their state, the way they're facing, etc.
//
UBYTE render()
{
	UBYTE i;
	WORD minusburlyspeedx = (-1)*burly.speedx.w;
	WORD minusburlyspeedy = (-1)*burly.speedy.w;

	// apply various transformations to the characters
	burly_render(&burly);

	// shift all foxes
	for(i=0;i<MAX_FOXES;i++)
	{
		// foxes move according to their own speed
		fox[i].x.w += fox[i].speedx.w;
		fox[i].y.w += fox[i].speedy.w;

		fox_render(&fox[i]);
	}

	cookie_render(&cookie);
	return 0u;
}

//
// Read D-pad and button input and change states and speeds accordingly
//
UBYTE handle_input( burly_t *burly, const UBYTE time )
{
	// Burly's speed is dependent on whether he has any active powerups
	WORD speedfactor = burly->standardSpeed;
	WORD minusspeedfactor = speedfactor*(-1);

	// read in the state of the D-pad and buttons
	UBYTE i = joypad();

	// when A is pressed, Burly stops and swipes
	// so we can short-circuit here
	if(i & J_A)
	{
		// re-seed in order to make stuff more "random"
		seed_random(time);

		if(burly->stateMotion != STATE_BURLY_ATTACKING)
		{
			// stop Burly from walking
			burly->stateMotion = STATE_BURLY_STOPPED;
			// he's now attacking
			burly->stateMotion = STATE_BURLY_ATTACKING;

			// short circuit
			return 0u;
		}
		else
		{
			// Burly is already attacking
			return 0u;
		}
	}

	if(i & J_SELECT)
	{
	}

	if(i & J_START)
	{
	}


	// Burly cannot start moving again until his swipe is finished
	if( burly->stateMotion != STATE_BURLY_ATTACKING)
	{

		if(i & J_UP)
		{
			burly->speedy.w = minusspeedfactor;
			burly->stateMotion = STATE_BURLY_WALKING;
		}
		else if(i & J_DOWN)
		{
			burly->speedy.w = speedfactor;
			burly->stateMotion = STATE_BURLY_WALKING;
		}
		else
		{
			burly->speedy.w = 0;
		}


		if(i & J_LEFT)
		{
			burly->speedx.w = minusspeedfactor;
			burly->state = STATE_BURLY_FACING_LEFT;
			burly->stateMotion = STATE_BURLY_WALKING;
		}
		else if(i & J_RIGHT)
		{		
			burly->speedx.w = speedfactor;
			burly->state = STATE_BURLY_FACING_RIGHT;
			burly->stateMotion = STATE_BURLY_WALKING;
		}
		else
		{
			burly->speedx.w = 0;
		}

		// did Burly stop?
		if( !(i & J_LEFT) && !(i & J_RIGHT) && !(i & J_UP) && !(i & J_DOWN) )
		{
			// this state is pretty much ignored by animation
			// Burly can be in this state only if D-pad is not pressed and he is not swiping
			burly->stateMotion = STATE_BURLY_STOPPED;
		}
	}

	return 0u;
}
