#include <rand.h>
#include "types.h"
#include "tiledata.h"
#include "palettes.h"

//
// initialize random number generator
//
// Note: make sure to attempt to seed with a different value before each game
//
// I do this by incrementing time rapidly during the main menu, since the time from when the
// ROM starts up until the time the player actually presses Start to begin the game, a 
// "random" number of increments have been applied to time
// It's probably not a bad idea to re-seed every now and then, such as when A is pressed,
// in order to make things even more "random"
//
int seed_random(UBYTE seed)
{
	seed = DIV_REG;
	seed |= (UWORD)DIV_REG << 8;
	initarand(seed);
	return 0;
}

// returns [0, n] (inclusive)
UBYTE next_random(UBYTE n)
{
	UBYTE r;
	r = rand();
	return r % (n + 1);
}

int burly_initialize( burly_t *burly )
{
	burly->x.w = 0x4800;
	burly->y.w = 0x4800;

	burly->speedx.w = 0x0000;
	burly->speedy.w = 0x0000;

	burly->currentFrame = 0;
	burly->totalFrames = ANIMATION_FRAMES_BURLY_WALK_LEFT_OR_RIGHT;

	burly->stateOld = STATE_BURLY_FACING_RIGHT;
	burly->state = STATE_BURLY_FACING_RIGHT;
	burly->stateMotion = STATE_BURLY_STOPPED;
	
	burly->walkRightFrames = tiles_burly_walk_right;
	burly->attackRightFrames = tiles_burly_attack_right;

	burly->powerupRemainingFrames = 0u;

	burly->firstSprite = 0;
	burly->secondSprite = 1;

	// have this value divide evenly into 0x1000 or you will see stuttering in
	// stationary objects due to rounding (while scrolling)
	burly->standardSpeed = BURLY_BASE_SPEED;

	return 0;
}

int fox_initialize( fox_t *fox, UBYTE first, UBYTE second )
{
	WORD sp;
	WORD baseSp = 0x0070u;
	UBYTE random;
	// adjust this formula as you see fit, to get a different fox distribution
	fox->x.w = -0x1800 + 0x0200 * next_random(100);
	fox->y.w = -0x1800 + 0x0200 * next_random(100);

	fox->speedx.w = 0x0000;
	fox->speedy.w = 0x0000;

	fox->currentFrame = 0;
	fox->totalFrames = ANIMATION_FRAMES_FOX_SPAWNING;

	fox->state = next_random(1)==1u ? STATE_FOX_FACING_RIGHT : STATE_FOX_FACING_LEFT;
	fox->stateOld = fox->state;
	fox->stateMotion = STATE_FOX_SPAWN;
	fox->stateMotionOld = STATE_FOX_SPAWN;

	//fox->standardSpeed = 0x0050;
	sp = 0x0005u;
	fox->standardSpeed = baseSp + next_random(30)*sp;

	fox->walkLeftFrames = tiles_fox_walk_left;
	fox->sittingLeftFrames = tiles_fox_sitting;
	fox->dyingFrames = tiles_fox_dying;
	fox->spawnFrames = tiles_fox_spawn;

	fox->firstSprite = first;
	fox->secondSprite = second;

	// assign a palette randomly
	random = next_random( 2 );
	if( random == 0 )
		fox->palette = PAL_FOX;
	else if( random == 1 )
		fox->palette = PAL_FOX2;
	else if( random == 2 )
		fox->palette = PAL_FOX3;
	
	return 0;
}

int cookie_initialize( cookie_t *cookie, UBYTE first )
{
	cookie->x.w = -0x1800 + 0x0200 * next_random(100);
	cookie->y.w = -0x1800 + 0x0200 * next_random(100);

	//hardcoded value -- good for testing
	//cookie->x.w = -0x1800 + 0x0200 * 60;
	//cookie->y.w = -0x1800 + 0x0200 * 50;

	cookie->frames = tiles_cookie[0];

	cookie->sprite = first;

	return 0;
}
